from typing import Any, Dict, Optional, Tuple, Union

import json
import os
import pathlib
import tempfile

import boto3
import brotli
import magic
from botocore.exceptions import ClientError

from .constants import MODE_TYPE_MAP
from .sentieo_aws_base import SentieoAWSBase


class SentieoS3Connection(SentieoAWSBase):
    """
    Class to connect to the S3 service.

    Attributes:
        aws_access_key_id (str): AWS access key ID.
        aws_secret_access_key (str): AWS secret access key.
        s3_config (str): S3 Config.
        local (bool): LocalService.

    """

    def __init__(
        self,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        s3_config: Optional[Dict[Any, Any]] = None,
        local: bool = False,
    ) -> None:
        super(SentieoS3Connection, self).__init__(
            "s3", aws_access_key_id, aws_secret_access_key, local
        )
        self.s3_config = s3_config

    def get_bucket(self, bucket_name: str) -> boto3.resource:
        """
        Get the S3 bucket
        Args:
            bucket_name (str): The name of the bucket.
        Returns:
            boto3.resource: The boto3 s3 bucket resource.
        """
        return self.aws_resource.Bucket(bucket_name)

    def get_object(self, bucket_name: str, object_name: str) -> boto3.resource:
        """
        Get the S3 object
        Args:
            bucket_name (str): The name of the bucket.
            object_name (str): The name of the object.
        Returns:
            boto3.resource: The boto3 s3 object resource.
        """
        return self.get_bucket(bucket_name).Object(object_name)

    def get_object_content(self, bucket_name: str, object_name: str) -> Tuple[str, str]:
        """
        Get the S3 object content
        Args:
            bucket_name (str): The name of the bucket.
            object_name (str): The name of the object.
        Returns:
            Tuple[str, str]: The content of the object.
        Raises:
            Exception: If the object does not exist.
        """
        try:
            s3_obj = self.get_object(bucket_name, object_name)
            content = s3_obj.get().get("Body").read()
            content_type = s3_obj.get().get("ContentType")
        except Exception as e:
            content, content_type = "", ""
            print(e)
        return content, content_type

    def put_object_content(
        self,
        bucket_name: str,
        object_name: str,
        content: Union[str, Dict[Any, Any]],
        content_type: str,
    ) -> bool:
        """
        Put the S3 object content
        Args:
            bucket_name (str): The name of the bucket.
            object_name (str): The name of the object.
            content (Union[str, dict]): The content of the object.
            content_type (str): The content type of the object.
        Returns:
            bool: True if the object was created.
        """
        try:
            s3_obj = self.get_object(bucket_name, object_name)
            if isinstance(content, dict):
                content = json.dumps(content)
            s3_obj.put(Body=content, ContentType=content_type)
            status = True
        except Exception as e:
            status = False
            print(e)
        return status

    def upload_file(
        self,
        bucket_name: str,
        object_name: str,
        file_path: str,
        compress: bool,
        make_public: bool = True,
        acl: str = "private",
    ) -> None:
        """
        Upload a file to S3
        Args:
            bucket_name (str): The name of the bucket.
            object_name (str): The name of the object.
            file_path (str): The path to the file.
            compress (bool): Whether to compress the file.
            make_public (bool): Whether to make the file public.
        Returns:
            None
        """
        mime = magic.Magic(mime=True)
        file_type = pathlib.Path(file_path).suffix
        extra_kwargs = {"ContentType": mime.from_file(file_path)}
        extra_kwargs.update(
            {"ACL": "public-read"}
        ) if make_public else extra_kwargs.update({"ACL": acl})

        bucket = self.get_bucket(bucket_name)
        if not compress:
            bucket.upload_file(file_path, object_name, ExtraArgs=extra_kwargs)
        else:
            tmp_file = tempfile.NamedTemporaryFile(
                mode="wb", suffix=".br", delete=False
            )
            with open(file_path, "rb") as f_in:
                file_content = f_in.read()
            if "font.css" in file_path:
                mode = brotli.MODE_FONT
            else:
                mode = MODE_TYPE_MAP.get(file_type, brotli.MODE_GENERIC)
            compressed_content = brotli.compress(file_content, mode=mode)
            with open(tmp_file.name, "wb") as c_file:
                c_file.write(compressed_content)
            extra_kwargs.update({"ContentEncoding": "br"})
            bucket.upload_file(tmp_file.name, object_name, ExtraArgs=extra_kwargs)
            os.unlink(tmp_file.name)

    def download_file(self, bucket_name: str, object_name: str, file_path: str) -> bool:
        """
        Download a file from S3
        Args:
            bucket_name (str): The name of the bucket.
            object_name (str): The name of the object.
            file_path (str): The path to the file.
        Returns:
            bool: True if the file was downloaded.
        """
        try:
            path = pathlib.Path(file_path)
            path.parent.mkdir(parents=True, exist_ok=True)
            self.get_object(bucket_name, object_name).download_file(file_path)
        except ClientError as e:
            print("AWS S3 Client Error", e)
            return False
        except FileNotFoundError as e:
            print("File Not Found", e)
            return False
        return True

    def get_pre_signed_url(
        self, bucket_name: str, object_name: str, expiration: int = 3600
    ) -> Any:
        """
        Generate a pre-signed url to share an S3 object
        Args:
            bucket_name (str): The name of the bucket
            object_name (str): The name of the object
            expiration (int): Time in seconds for the presigned URL to remain valid
        Returns:
            str: Presigned URL as string. If error, returns None.

        """
        try:
            response = self.aws_client.generate_presigned_url(
                "get_object",
                Params={"Bucket": bucket_name, "Key": object_name},
                ExpiresIn=expiration,
            )
        except ClientError as e:
            print("Error creating the pre signed url ", e)
            return None
        return response



from typing import Any, Dict, Optional

import boto3

from .sentieo_aws_base import SentieoAWSBase


class SentieoSQSConnection(SentieoAWSBase):
    """
    Class to connect to SQS service.

    Attributes:
        aws_access_key_id (str): AWS access key ID.
        aws_secret_access_key (str): AWS secret access key.
        local (bool): LocalService.

    """

    def __init__(
        self,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        local: bool = False,
    ) -> None:
        super(SentieoSQSConnection, self).__init__(
            "sqs", aws_access_key_id, aws_secret_access_key, local
        )

    def get_queue(self, queue_name) -> Optional[boto3.resource]:
        """
        Get an sqs Queue
        Args:
            queue_name (str): name of the sqs queue
        Returns:
            queue_object
        """
        try:
            queue = self.aws_resource.get_queue_by_name(QueueName=queue_name)
        except Exception:
            queue = None

        return queue

    def send_message(self, queue_name, message_body) -> Dict[str, Any]:
        """
        Send a message to the specified sqs Queue
        Args:
            queue_name (str): name of the sqs queue
            message_body (str): message being sent to the queue
        Returns:
            JSON response consisting of request details & response
        """
        queue = self.get_queue(queue_name)

        return queue.send_message(MessageBody=message_body)

    def read_message(self, queue_name) -> Dict[str, Any]:
        """
        Reads a message from specified queue
        Args:
            queue_name (str): name of the sqs queue
        Returns:
            JSON response consisting of message details
        """
        queue = self.get_queue(queue_name)

        return self.aws_client.receive_message(
            QueueUrl=queue.url, MaxNumberOfMessages=1, AttributeNames=["All"]
        )

    def delete_message(self, queue_name, receipt_handle) -> Dict[str, Any]:
        """
        Deletes the specified message from specified queue
        Args:
            queue_name (str): name of the sqs queue
            receipt_handle (str): receipt_handle of the message to be deleted
        Returns:
            JSON response consisting of request & response details
        """
        queue = self.get_queue(queue_name)

        return self.aws_client.delete_message(
            QueueUrl=queue.url, ReceiptHandle=receipt_handle
        )



from typing import Optional

import boto3
import localstack_client.session as local_boto3
from botocore.client import Config
from botocore.exceptions import ClientError


class SentieoAWSBase:
    """
    Class to connect to the AWS service(s).

    Attributes:
        aws_access_key_id (str): AWS access key ID.
        aws_secret_access_key (str): AWS secret access key.
        local (bool): LocalService.

    """

    def __init__(
        self,
        aws_service: str,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        local: bool = False,
    ) -> None:
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.local = local
        self.aws_service = aws_service
        self.aws_client = self.get_client()
        self.aws_resource = self.get_resource()

    def get_client(self) -> boto3.client:
        """
        Get the boto3 client for the S3 service.
        Returns:
            boto3.client: The boto3 client for the AWS service.
        Raises:
            Exception: If the client could not be created.
        """
        try:
            if self.local:
                aws_client = local_boto3.client(self.aws_service)
            elif self.aws_access_key_id and self.aws_secret_access_key:
                aws_client = boto3.client(
                    self.aws_service,
                    aws_access_key_id=self.aws_access_key_id,
                    aws_secret_access_key=self.aws_secret_access_key,
                )
            else:
                if self.aws_service == "s3":
                    aws_client = boto3.client(
                        self.aws_service, config=Config(signature_version="s3v4")
                    )
                else:
                    aws_client = boto3.client(self.aws_service)
        except ClientError as e:
            print(f"Exception occurred : {e}")
            aws_client = None
        return aws_client

    def get_resource(self) -> boto3.resource:
        """
        Get the boto3 resource for the AWS service.
        Returns:
            boto3.resource: The boto3 resource for the AWS service.
        Raises:
            Exception: If the resource could not be created.
        """
        try:
            if self.local:
                aws_resource = local_boto3.resource(self.aws_service)
            elif self.aws_access_key_id and self.aws_secret_access_key:
                aws_resource = boto3.resource(
                    self.aws_service,
                    aws_access_key_id=self.aws_access_key_id,
                    aws_secret_access_key=self.aws_secret_access_key,
                )
            else:
                if self.aws_service == "s3":
                    aws_resource = boto3.resource(
                        self.aws_service, config=Config(signature_version="s3v4")
                    )
                else:
                    aws_resource = boto3.resource(self.aws_service)
        except ClientError as e:
            print(f"Exception occurred : {e}")
            aws_resource = None

        return aws_resource



from typing import Any, Dict, List, Optional

from .sentieo_aws_base import SentieoAWSBase


class SentieoSNSConnection(SentieoAWSBase):
    """
    Class to connect to SNS service.

    Attributes:
        aws_access_key_id (str): AWS access key ID.
        aws_secret_access_key (str): AWS secret access key.
        local (bool): LocalService.

    """

    def __init__(
        self,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        local: bool = False,
    ) -> None:
        super(SentieoSNSConnection, self).__init__(
            "sns", aws_access_key_id, aws_secret_access_key, local
        )

    def get_all_topics(self) -> List[str]:
        """
        Get all SNS topics
        Args:
            None
        Returns:
            returns a list of all topic names
        """
        resp = self.aws_client.list_topics()

        return [topic["TopicArn"] for topic in resp["Topics"]]

    def get_topic_arn(self, topic_name) -> Optional[str]:
        """
        Get the topic ARN
        Args:
            topic_name (str): name of the sns topic
        Returns:
            ARN of the topic
        """
        if "arn:aws:sns" in topic_name:
            return topic_name

        for topic in self.get_all_topics():
            if topic_name in topic:
                return topic

        return None

    def create_topic(self, topic_name) -> Dict[str, Any]:
        """
        Creates an SNS topic
        Args:
            topic_name (str): name of the sns topic
        Returns:
            JSON response consisting of created topic details
        """
        return self.aws_client.create_topic(Name=topic_name)

    def publish(self, msg, topic_name) -> Dict[str, Any]:
        """
        Publishes a message to the specified sns topic
        Args:
            topic_name (str): name of the sns topic to which msg is being published
            msg (str): message which is being published to the specified sns topic
        Returns:
            JSON response consisting of published message details
        """
        topic_arn = self.get_topic_arn(topic_name)

        return self.aws_client.publish(Message=msg, TopicArn=topic_arn)

    def subscribe(self, topic_name, protocol, endpoint) -> Dict[str, Any]:
        """
        Subscribes to the specified topic, with the desired protocol & endpoint
        Args:
            topic_name (str): name of the sns topic which is being subscribed to
            protocol (str): email, phone etc
            endpoint (str): email value, phone number etc
        Returns:
            JSON response consisting of subscription details
        """
        topic_arn = self.get_topic_arn(topic_name)

        return self.aws_client.subscribe(
            TopicArn=topic_arn, Protocol=protocol, Endpoint=endpoint
        )

    def subscribe_sqs_queue(self, topic_name, endpoint) -> Dict[str, Any]:
        """
        Subscribes an sqs queue to topic_name
        Args:
            topic_name (str): name of the sns topic which is being subscribed to
            endpoint (str): ARN or URL of the sqs queue
        Returns:
            JSON response consiting of subscription details
        """
        topic_arn = self.get_topic_arn(topic_name)

        return self.subscribe(topic_arn, "sqs", endpoint)
